#include "Objects.h"
#include <iostream>
using namespace std;

const double PI = 3.14153;

int Object::last_id = 0;

Object::Object() {
	this->id = ++last_id;
}

Object::~Object() {}

void Object::print() {
	cout << this->label() << this->id << "(" << this->v() << ")";
}

int Object::getId() {
	return this->id;
}

Cube::Cube(double a) {
	this->a = a;
}

double Cube::v() {
	return this->a * this->a * this->a;
}

char Cube::label() {
	return 'C';
}

Sphere::Sphere(double r) {
	this->r = r;
}

double Sphere::v() {
	return 4.0 / 3.0 * this->r * this->r * this->r * PI;
}

char Sphere::label() {
	return 'S';
}

ComplexObject::ComplexObject(string object) {
	this->first = this->last = nullptr;
	int number = 0;
	for (int i = 0; i < object.length(); i++) {
		Object* obj = nullptr;
		switch (object[i]) {
		case 'C':
			addObject(new Cube(number));
			number = 0;
			break;
		case 'S':
			addObject(new Sphere(number));
			number = 0;
			break;
		case '1': case '2': case '3': case '4': case '5':
		case '6': case '7': case '8': case '9': case '0':
			number = number * 10 + (object[i] - '0');
			break;
		default:
			break;
		}
	}
}

ComplexObject::~ComplexObject() {}

double ComplexObject::v() {
	Elem* curr = this->first;
	double vol = 0.0;
	while (curr) {
		vol += curr->object->v();
		curr = curr->next;
	}
	return vol;
}

char ComplexObject::label() {
	return 'X';
}

void ComplexObject::print() {
	Object::print();
	cout << '[';
	Elem* curr = this->first;
	while (curr) {
		cout << curr->object->label();
		curr = curr->next;
	}
	cout << ']';
}

ObjectList::ObjectList() {
	this->first = this->last = nullptr;
	objects_count = 0;
}

ObjectList::~ObjectList() {
	while (this->first) {
		Elem* curr = this->first;
		this->first = this->first->next;
		delete curr->object;
		delete curr;
	}
	this->last = nullptr;
	objects_count = 0;
}

void ObjectList::addObject(Object* object) {
	if (!this->first) {
		this->first = new Elem(object);
		this->last = this->first;
	}
	else {
		this->last->next = new Elem(object);
		this->last = this->last->next;
	}
	objects_count++;
}

int ObjectList::numberOfObjects() {
	return objects_count;
}

Object* ObjectList::removeFirst() {
	if (this->first) {
		Elem* curr = this->first;
		Object* obj = this->first->object;
		if (this->last == this->first) this->last = nullptr;
		this->first = this->first->next;
		delete curr; objects_count--;
		return obj;
	}
	return nullptr;
}

Object* ObjectList::removeLast() {
	if (this->last) {
		Elem* curr = this->first;
		while (curr && curr->next != this->last) {
			curr = curr->next;
		}
		Object* obj = nullptr;
		if (!curr) {
			curr = this->first;
			obj = curr->object;
			this->first = this->last = nullptr;
		}
		else {
			obj = this->last->object;
			this->last = curr;
			curr = curr->next;
		}
		delete curr; objects_count--;
		return obj;
	}
	return nullptr;
}
