#ifndef OBJECTS_H
#define OBJECTS_H

class Object {
public:
	Object();
	// Virtuelan destruktor
	// Poziv destruktora operatorom delete poziva onaj destruktor koji odgovara pokazivanom objektu
	// Da destruktor nije bio virtuelan, pozvao bi se destruktor iz klase koja odgovara pokazivacu
	// To moze da dovede do curenja memorije
	virtual ~Object();
	// Metoda za koju se ne zna implementacija se oznacava kao apstraktna (ili cisto virtuelna) dodavanjem = 0 u nastavku deklaracije metode.
	// Klase sa makar jednom apstraktnom metodom su i same apstraktne
	// Ne mozemo kreirati objekte apstraktnih klasa
	// Object* o = new Object; - prevodilac javlja gresku
	// Apstraktne klase sluze da grupisemo srodne klase (kocka, sfera, ...) u jednu osnovnu klasu (telo), 
	// predvidimo postojanje neke metode (zapremina) za koju ce postojati konkretne implementacije u izvedenim klasama.
	// Na ovaj nacin mozemo imati kolekcije objekata izvedenih klasa, tako sto imamo npr. niz pokazivaca na Object, pa mozemo da koristimo polimorfizam.
	virtual double v() = 0;
	virtual char label() = 0;
	virtual void print();
	int getId();

private:
	int id;
	static int last_id;
};

class Cube : public Object {
public:
	Cube(double a);
	// override - naznaka prevodiocu da je metoda nadjacanje metode iz osnovne klase
	// prevodilac javlja gresku ukoliko metoda sa datim potpisom ne postoji u osnovnoj klasi ili nije virtuelna
	double v() override;
	char label() override;

private:
	double a;
};

class Sphere : public Object {
public:
	Sphere(double r);
	double v() override;
	char label() override;

private:
	double r;
};

#include <string>
using namespace std;

class ObjectList {
public:
	ObjectList();
	virtual ~ObjectList();

	virtual void addObject(Object* object);
	int numberOfObjects();
	Object* removeFirst();
	Object* removeLast();

protected:
	struct Elem {
		Elem(Object* object) {
			this->object = object;
			this->next = nullptr;
		}
		Object* object;
		Elem* next;
	};

	Elem* first, * last;
	int objects_count;
};

class ComplexObject : public Object, public ObjectList {
public:
	ComplexObject(string object);
	~ComplexObject();

	double v() override;
	char label() override;
	void print() override;
};

#endif // !FIGURES_H
